#include "led.h"

#include "board_config.h"
//#include "elog.h"
//#include "global_timer.h"

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

static GPIO_TypeDef *const LED_PORT[LED_END] = { LED1_PORT, LED2_PORT };
static const uint16_t LED_PIN[LED_END] = { LED1_PIN, LED2_PIN };

static inline bool IsValid(LED_t led);

void LED_Init(void)
{
    static const uint32_t LED_GPIO_APB2_PERIPH[LED_END] = {
        LED1_GPIO_APB2_PERIPH,
        LED2_GPIO_APB2_PERIPH,
    };
    for (LED_t led = LED1; led != LED_END; ++led) {
        RCC_APB2PeriphClockCmd(LED_GPIO_APB2_PERIPH[led], ENABLE);
        GPIO_InitTypeDef gpio_init_struct = { 0 };
        gpio_init_struct.GPIO_Mode = GPIO_Mode_Out_PP;
        gpio_init_struct.GPIO_Pin = LED_PIN[led];
        gpio_init_struct.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_Init(LED_PORT[led], &gpio_init_struct);
        LED_Off(led);
    }
}

void LED_On(LED_t led)
{
    if (IsValid(led)) {
        GPIO_ResetBits(LED_PORT[led], LED_PIN[led]);
    }
}

void LED_Off(LED_t led)
{
    if (IsValid(led)) {
        GPIO_SetBits(LED_PORT[led], LED_PIN[led]);
    }
}

void LED_Toggle(LED_t led)
{
    if (IsValid(led)) {
        bool on = (Bit_RESET
                   == GPIO_ReadOutputDataBit(LED_PORT[led], LED_PIN[led]));
        if (on) {
            LED_Off(led);
        } else {
            LED_On(led);
        }
    }
}

void LED_Test_Task10ms(void)
{
    static bool m_initialized = false;
    if (!m_initialized) {
        m_initialized = true;
        for (LED_t led = (LED_t)0u; led != LED_END; ++led) {
            if (((uint32_t)led % 2u) == 0u) {
                LED_On(led);
            } else {
                LED_Off(led);
            }
        }
    }
    // static GT_Time_t m_pre_time = 0u;
    // if (GT_CheckTimeoutAndUpdateTimeHolder(&m_pre_time, 1000u)) {
    for (LED_t led = (LED_t)0u; led != LED_END; ++led) {
        LED_Toggle(led);
    }
    // }
}

void LED_Test_InfiniteLoop(void)
{
    while (true) {
        volatile uint32_t i = 5000000u;
        while (i > 0u) {
            i--;
        }
        LED_On(LED1);
        LED_Off(LED2);
        i = 5000000u;
        while (i > 0u) {
            i--;
        }
        LED_Off(LED1);
        LED_On(LED2);
    }
}

static inline bool IsValid(LED_t led)
{
    // Convert to size_t before comparing to avoid signedness issues
    return ((size_t)led < (size_t)LED_END);
}
